@@ -36,7 +36,7 @@ import Control.Monad.IO.Class (liftIO)
 
 import Data.ByteString.Builder (toLazyByteString)
 
-import Data.FileEmbed (embedFile)
+import Data.FileEmbed (embedFile, makeRelativeToProject)
 
 import Data.Text (Text)
 import qualified Data.Text as T
@@ -74,7 +74,7 @@ filterButLast f (x:xs)
 
 -- | Serve an appropriate response for a folder request.
 serveFolder :: StaticSettings -> Pieces -> W.Request -> Folder -> IO StaticResponse
-serveFolder StaticSettings {..} pieces req folder@Folder {..} =
+serveFolder StaticSettings {..} pieces req folder =
     case ssListing of
         Just _ | Just path <- addTrailingSlash req, ssAddTrailingSlash ->
             return $ RawRedirect path
@@ -164,11 +164,13 @@ serveFile StaticSettings {..} req file
             -- Didn't match, but we have a hash value. Send the file contents
             -- with an ETag header.
             --
-            -- Note: It would be arguably better to next check
-            -- if-modified-since and return a 304 if that indicates a match as
-            -- well. However, the circumstances under which such a situation
-            -- could arise would be very anomalous, and should likely warrant a
-            -- new file being sent anyway.
+            -- RFC7232 (HTTP 1.1):
+            -- > A recipient MUST ignore If-Modified-Since if the request contains an
+            -- > If-None-Match header field; the condition in If-None-Match is
+            -- > considered to be a more accurate replacement for the condition in
+            -- > If-Modified-Since, and the two are only combined for the sake of
+            -- > interoperating with older intermediaries that might not implement
+            -- > If-None-Match.
             (Just hash, _) -> respond [("ETag", hash)]
 
             -- No hash value available, fall back to last modified support.
@@ -203,23 +205,22 @@ cacheControl :: MaxAge -> (H.ResponseHeaders -> H.ResponseHeaders)
 cacheControl maxage =
     headerCacheControl . headerExpires
   where
-    ccInt =
-        case maxage of
-            NoMaxAge -> Nothing
-            MaxAgeSeconds i -> Just i
-            MaxAgeForever -> Just oneYear
     oneYear :: Int
     oneYear = 60 * 60 * 24 * 365
 
-    headerCacheControl =
-      case ccInt of
-        Nothing -> id
-        Just i  -> (:) ("Cache-Control", S8.append "public, max-age=" $ S8.pack $ show i)
+    maxAgeValue i = S8.append "public, max-age=" $ S8.pack $ show i
+
+    headerCacheControl = case maxage of
+        NoMaxAge -> id
+        MaxAgeSeconds i -> (:) ("Cache-Control", maxAgeValue i)
+        MaxAgeForever -> (:) ("Cache-Control", maxAgeValue oneYear)
+        NoStore -> (:) ("Cache-Control", "no-store")
     headerExpires =
       case maxage of
         NoMaxAge        -> id
         MaxAgeSeconds _ -> id -- FIXME
         MaxAgeForever   -> (:) ("Expires", "Thu, 31 Dec 2037 23:55:55 GMT")
+        NoStore -> id
 
 -- | Turn a @StaticSettings@ into a WAI application.
 staticApp :: StaticSettings -> W.Application
@@ -231,8 +232,8 @@ staticAppPieces _ _ req sendResponse
         H.status405
         [("Content-Type", "text/plain")]
         "Only GET or HEAD is supported"
-staticAppPieces _ [".hidden", "folder.png"] _ sendResponse = sendResponse $ W.responseLBS H.status200 [("Content-Type", "image/png")] $ L.fromChunks [$(embedFile "images/folder.png")]
-staticAppPieces _ [".hidden", "haskell.png"] _ sendResponse = sendResponse $ W.responseLBS H.status200 [("Content-Type", "image/png")] $ L.fromChunks [$(embedFile "images/haskell.png")]
+staticAppPieces _ [".hidden", "folder.png"] _ sendResponse = sendResponse $ W.responseLBS H.status200 [("Content-Type", "image/png")] $ L.fromChunks [$(makeRelativeToProject "images/folder.png" >>= embedFile)]
+staticAppPieces _ [".hidden", "haskell.png"] _ sendResponse = sendResponse $ W.responseLBS H.status200 [("Content-Type", "image/png")] $ L.fromChunks [$(makeRelativeToProject "images/haskell.png" >>= embedFile)]
 staticAppPieces ss rawPieces req sendResponse = liftIO $ do
     case toPieces rawPieces of
         Just pieces -> checkPieces ss pieces req >>= response
@@ -243,7 +244,7 @@ staticAppPieces ss rawPieces req sendResponse = liftIO $ do
     response :: StaticResponse -> IO W.ResponseReceived
     response (FileResponse file ch) = do
         mimetype <- ssGetMimeType ss file
-        let filesize = fileGetSize file
+        -- let filesize = fileGetSize file
         let headers = ("Content-Type", mimetype)
                     -- Let Warp provide the content-length, since it takes
                     -- range requests into account
@@ -278,7 +279,7 @@ staticAppPieces ss rawPieces req sendResponse = liftIO $ do
                 , ("Location", path)
                 ] "Redirect"
 
-    response NotFound = case (ss404Handler ss) of
+    response NotFound = case ss404Handler ss of
         Just app -> app req sendResponse
         Nothing  -> sendResponse $ W.responseLBS H.status404
                         [ ("Content-Type", "text/plain")
