@@ -1,6 +1,6 @@
 import WaiAppStaticTest (spec)
 import WaiAppEmbeddedTest (embSpec)
-import Test.Hspec.Monadic
+import Test.Hspec
 
 main :: IO ()
 main = hspec $ spec >> embSpec
